"""
SplitLunch Configuration Helpers
"""


class SplitLunchConfig:
    """
    Configuration Namespace for SplitLunch
    """

    splitlunch_tag: str = "SplitLunch"
    splitwise_tag: str = "Splitwise"
    splitlunch_import_tag: str = "SplitLunchImport"
    splitlunch_direct_import_tag: str = "SplitLunchDirectImport"
